package org.openxma.dsl.common.formatter;

import java.util.ArrayList;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.AbstractRule;
import org.eclipse.xtext.RuleCall;
import org.eclipse.xtext.parsetree.AbstractNode;
import org.openxma.dsl.common.formatter.locator.SimpleLocator;

public class LocatorTree {
	
	private ArrayList<SimpleLocator> around = new ArrayList<SimpleLocator>();
	private ArrayList<SimpleLocator> before = new ArrayList<SimpleLocator>();
	private ArrayList<SimpleLocator> after = new ArrayList<SimpleLocator>();
	private ArrayList<SimpleLocator> between = new ArrayList<SimpleLocator>();
	private ArrayList<SimpleLocator> within = new ArrayList<SimpleLocator>();	
	
	/**
	 * Find the first matching Locator element
	 * 
	 * @param elementOfLeftNode
	 * @param elementOfRightNode
	 * @return
	 */
	public SimpleLocator findFirstLocator(AbstractNode node, EObject elementOfLeftNode, EObject elementOfRightNode) {
		for (SimpleLocator locator : before) {
			if (isBefore(elementOfRightNode, locator)) {
				return locator;
			}								
		}
		for (SimpleLocator locator : after) {
			if (isAfter(elementOfLeftNode, locator)) {
				return locator;
			}								
		}
		for (SimpleLocator locator : around) {
			if (isAround(elementOfLeftNode, elementOfRightNode, locator)) {
				return locator;
			}								
		}
		for (SimpleLocator locator : between) {
			if (isBetween(elementOfLeftNode, elementOfRightNode, locator)) {
				return locator;
			}								
		}
		for (SimpleLocator locator : within) {
			if (isWithin(node,elementOfLeftNode, elementOfRightNode, locator)) {
				return locator;
			}								
		}				
		return null;			
	}	
	
	/** 
	 * Find all matching Locator elements
	 * @param elementOfLeftNode
	 * @param elementOfRightNode
	 * @return
	 */
	public ArrayList<SimpleLocator> findAllLocators(AbstractNode node,EObject elementOfLeftNode, EObject elementOfRightNode) {
		ArrayList<SimpleLocator> foundLocators = new ArrayList<SimpleLocator>();						
		for (SimpleLocator locator : around) {
			if (isAround(elementOfLeftNode, elementOfRightNode, locator)) {
				foundLocators.add(locator);
			}								
		}
		for (SimpleLocator locator : before) {
			if (isBefore(elementOfRightNode, locator)) {
				foundLocators.add(locator);
			}								
		}
		for (SimpleLocator locator : after) {
			if (isAfter(elementOfLeftNode, locator)) {
				foundLocators.add(locator);
			}								
		}
		for (SimpleLocator locator : between) {
			if (isBetween(elementOfLeftNode, elementOfRightNode, locator)) {
				foundLocators.add(locator);
			}								
		}
		for (SimpleLocator locator : within) {
			if (isWithin(node,elementOfLeftNode, elementOfRightNode, locator)) {
				foundLocators.add(locator);
			}								
		}		
		return foundLocators;			
	}
	
	/**
	 * Determine wheater at least one Locator matches
	 * 
	 * @param elementOfLeftNode
	 * @param elementOfRightNode
	 * @param locator
	 * @return
	 */
	public boolean isLocatorDefined(AbstractNode node, EObject elementOfLeftNode, EObject elementOfRightNode) {
		if (findFirstLocator(node,elementOfLeftNode,elementOfRightNode) != null) {
			return true;
		}
		return false;
	}		

	private boolean isBetween(EObject elementOfLeftNode,
			EObject elementOfRightNode, SimpleLocator locator) {
		return isAfter(elementOfLeftNode, locator) && isBefore(elementOfRightNode, locator);
	}

	private boolean isAfter(EObject elementOfLeftNode, SimpleLocator locator) {
		return elementOfLeftNode == locator.getLeft();
	}

	private boolean isBefore(EObject elementOfRightNode, SimpleLocator locator) {
		return elementOfRightNode == locator.getRight();
	}

	private boolean isAround(EObject elementOfLeftNode,	EObject elementOfRightNode, SimpleLocator locator) {
		return isAfter(elementOfLeftNode, locator) || isBefore(elementOfRightNode, locator);
	}
	
	private boolean isWithin(AbstractNode node, EObject elementOfLeftNode,	EObject elementOfRightNode, SimpleLocator locator) {
		EObject container = locator.getLeft();
		AbstractNode parent = node;		
		while (parent != null) {
			EObject grammarElement = parent.getGrammarElement();
			AbstractRule rule = OpenXMANodeModelFormatter.getRuleOfRuleCall(grammarElement);			
			if (grammarElement == container || rule == container) {
				return true;
			}
			parent = (AbstractNode)parent.eContainer();
		}
		return false;
	}	

	public void addAround(SimpleLocator locator) {
		around.add(locator);		
	}

	public void addAfter(SimpleLocator locator) {
		after.add(locator);			
	}

	public void addBefore(SimpleLocator locator) {
		before.add(locator);			
	}

	public void addBetween(SimpleLocator locator) {
		between.add(locator);			
	}
	
	public void addWithin(SimpleLocator locator) {
		within.add(locator);
	}
}