package org.openxma.dsl.common.syntaxcoloring;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.RuleCall;
import org.eclipse.xtext.parsetree.AbstractNode;
import org.eclipse.xtext.parsetree.CompositeNode;
import org.eclipse.xtext.parsetree.LeafNode;
import org.eclipse.xtext.parsetree.NodeAdapter;
import org.eclipse.xtext.parsetree.NodeUtil;
import org.eclipse.xtext.resource.XtextResource;
import org.eclipse.xtext.ui.editor.syntaxcoloring.IHighlightedPositionAcceptor;
import org.eclipse.xtext.ui.editor.syntaxcoloring.ISemanticHighlightingCalculator;

abstract public class OpenXMASemanticHighlightingCalculator implements ISemanticHighlightingCalculator {

	abstract public void provideHighlightingFor(XtextResource resource, IHighlightedPositionAcceptor acceptor);

	protected AbstractNode getFirstRuleCallNode(EObject semantic, String rule) {
		NodeAdapter adapter = NodeUtil.getNodeAdapter(semantic);
		if (adapter != null) {
			CompositeNode node = adapter.getParserNode();
			if (node != null) {
				if (rule == null) {
					return node;
				}
				return getRuleNode(rule, node);
			}
		}
		return null;
	}

	private AbstractNode getRuleNode(String rule, CompositeNode node) {
		for (AbstractNode child : node.getChildren()) {
			if (child.getGrammarElement() instanceof RuleCall
					&& ((RuleCall) child.getGrammarElement()).getRule().getName().equals(rule)) {
				return child;
			}
			if (child instanceof CompositeNode) {
				return getRuleNode(rule, (CompositeNode) child);
			}
		}
		return null;
	}

	protected AbstractNode getFirstFeatureNode(EObject semantic, String feature) {
		NodeAdapter adapter = NodeUtil.getNodeAdapter(semantic);
		if (adapter != null) {
			CompositeNode node = adapter.getParserNode();
			if (node != null) {
				if (feature == null) {
					return node;
				}
				return getFeatureNode(feature, node);
			}
		}
		return null;
	}

	private AbstractNode getFeatureNode(String feature, CompositeNode node) {
		for (AbstractNode child : node.getChildren()) {
			if (child instanceof LeafNode) {
				if (feature.equals(((LeafNode) child).getFeature())) {
					return child;
				}
			} else if (child instanceof CompositeNode) {
				return getFeatureNode(feature, (CompositeNode) child);
			}
		}
		return null;
	}

	protected void highlightNode(AbstractNode node, String id, IHighlightedPositionAcceptor acceptor) {
		if (node == null)
			return;
		if (node instanceof LeafNode) {
			acceptor.addPosition(node.getOffset(), node.getLength(), id);
		} else {
			for (LeafNode leaf : node.getLeafNodes()) {
				if (!leaf.isHidden()) {
					acceptor.addPosition(leaf.getOffset(), leaf.getLength(), id);
				}
			}
		}
	}

}
